@@ -22,6 +22,7 @@
 // MLT Header files
 #include <framework/mlt_producer.h>
 #include <framework/mlt_frame.h>
+#include <framework/mlt_profile.h>
 
 // ffmpeg Header files
 #include <avformat.h>
@@ -39,6 +40,10 @@
 #include <pthread.h>
 #include <math.h>
 
+#if LIBAVUTIL_VERSION_INT < (50<<16)
+#define PIX_FMT_YUYV422 PIX_FMT_YUV422
+#endif
+
 void avformat_lock( );
 void avformat_unlock( );
 
@@ -629,8 +634,8 @@ static inline void convert_image( AVFrame *frame, uint8_t *buffer, int pix_fmt,
 	else
 	{
 		AVPicture output;
-		avpicture_fill( &output, buffer, PIX_FMT_YUV422, width, height );
-		img_convert( &output, PIX_FMT_YUV422, (AVPicture *)frame, pix_fmt, width, height );
+		avpicture_fill( &output, buffer, PIX_FMT_YUYV422, width, height );
+		img_convert( &output, PIX_FMT_YUYV422, (AVPicture *)frame, pix_fmt, width, height );
 	}
 #endif
 }
