@@ -24,6 +24,7 @@
 extern mlt_consumer consumer_null_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_brightness_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_channelcopy_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
+extern mlt_filter filter_crop_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_data_feed_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_data_show_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
 extern mlt_filter filter_gamma_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
@@ -51,6 +52,7 @@ MLT_REPOSITORY
 	MLT_REGISTER( consumer_type, "null", consumer_null_init );
 	MLT_REGISTER( filter_type, "brightness", filter_brightness_init );
 	MLT_REGISTER( filter_type, "channelcopy", filter_channelcopy_init );
+	MLT_REGISTER( filter_type, "crop", filter_crop_init );
 	MLT_REGISTER( filter_type, "data_feed", filter_data_feed_init );
 	MLT_REGISTER( filter_type, "data_show", filter_data_show_init );
 	MLT_REGISTER( filter_type, "gamma", filter_gamma_init );
