@@ -18,8 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "filter_resize.h"
-
+#include <framework/mlt_filter.h>
 #include <framework/mlt_frame.h>
 
 #include <stdio.h>
@@ -53,6 +52,13 @@ static int filter_get_image( mlt_frame this, uint8_t **image, mlt_image_format *
 	// Retrieve the aspect ratio
 	double aspect_ratio = mlt_deque_pop_back_double( MLT_FRAME_IMAGE_STACK( this ) );
 
+	// Correct Width/height if necessary
+	if ( *width == 0 || *height == 0 )
+	{
+		*width = mlt_properties_get_int( properties, "normalised_width" );
+		*height = mlt_properties_get_int( properties, "normalised_height" );
+	}
+
 	// Assign requested width/height from our subordinate
 	int owidth = *width;
 	int oheight = *height;
@@ -83,22 +89,23 @@ static int filter_get_image( mlt_frame this, uint8_t **image, mlt_image_format *
 		double input_ar = aspect_ratio * real_width / real_height;
 		double output_ar = mlt_properties_get_double( properties, "consumer_aspect_ratio" ) * owidth / oheight;
 		
-		//fprintf( stderr, "normalised %dx%d output %dx%d %f %f\n", normalised_width, normalised_height, owidth, oheight, ( float )output_ar, ( float )mlt_properties_get_double( properties, "consumer_aspect_ratio" ) * owidth / oheight );
+// 		fprintf( stderr, "real %dx%d normalised %dx%d output %dx%d sar %f in-dar %f out-dar %f\n",
+// 		real_width, real_height, normalised_width, normalised_height, owidth, oheight, aspect_ratio, input_ar, output_ar);
 
 		// Optimised for the input_ar > output_ar case (e.g. widescreen on standard)
-		int scaled_width = rint( 0.5 + ( input_ar * normalised_width ) / output_ar );
+		int scaled_width = rint( ( input_ar * normalised_width ) / output_ar );
 		int scaled_height = normalised_height;
 
 		// Now ensure that our images fit in the output frame
 		if ( scaled_width > normalised_width )
 		{
 			scaled_width = normalised_width;
-			scaled_height = rint( 0.5 + ( output_ar * normalised_height ) / input_ar );
+			scaled_height = rint( ( output_ar * normalised_height ) / input_ar );
 		}
 
 		// Now calculate the actual image size that we want
-		owidth = rint( 0.5 + scaled_width * owidth / normalised_width );
-		oheight = rint( 0.5 + scaled_height * oheight / normalised_height );
+		owidth = rint( scaled_width * owidth / normalised_width );
+		oheight = rint( scaled_height * oheight / normalised_height );
 
 		// Tell frame we have conformed the aspect to the consumer
 		mlt_frame_set_aspect_ratio( this, mlt_properties_get_double( properties, "consumer_aspect_ratio" ) );
@@ -121,7 +128,7 @@ static int filter_get_image( mlt_frame this, uint8_t **image, mlt_image_format *
 
 		// Provides a manual override for misreported field order
 		if ( mlt_properties_get( properties, "meta.top_field_first" ) )
-			mlt_properties_set( properties, "top_field_first", mlt_properties_get_int( properties, "meta.top_field_first" ) );
+			mlt_properties_set_int( properties, "top_field_first", mlt_properties_get_int( properties, "meta.top_field_first" ) );
 
 		// Correct field order if needed
 		if ( mlt_properties_get_int( properties, "top_field_first" ) == 1 )
@@ -130,22 +137,8 @@ static int filter_get_image( mlt_frame this, uint8_t **image, mlt_image_format *
 			int size;
 			uint8_t *image = mlt_properties_get_data( properties, "image", &size );
 			uint8_t *ptr = image + owidth * 2;
-			int h = oheight / 2;
-			int w = owidth;
-
-			// Swap the lines around
-			while( h -- )
-			{
-				w = owidth;
-				while( w -- )
-				{
-					swap_bytes( image ++, ptr ++ );
-					swap_bytes( image ++, ptr ++ );
-				}
-				image += owidth * 2;
-				ptr += owidth * 2;
-			}
-
+			memmove( ptr, image, size - owidth * 2 );
+			
 			// Set the normalised field order
 			mlt_properties_set_int( properties, "top_field_first", 0 );
 			mlt_properties_set_int( properties, "meta.top_field_first", 0 );
@@ -189,7 +182,7 @@ static mlt_frame filter_process( mlt_filter this, mlt_frame frame )
 /** Constructor for the filter.
 */
 
-mlt_filter filter_resize_init( char *arg )
+mlt_filter filter_resize_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg )
 {
 	mlt_filter this = calloc( sizeof( struct mlt_filter_s ), 1 );
 	if ( mlt_filter_init( this, this ) == 0 )
