@@ -18,10 +18,12 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "producer_libdv.h"
+#include <framework/mlt_producer.h>
 #include <framework/mlt_frame.h>
 #include <framework/mlt_deque.h>
 #include <framework/mlt_factory.h>
+#include <framework/mlt_profile.h>
+
 #include <pthread.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,6 +34,9 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#define FRAME_SIZE_525_60 	10 * 150 * 80
+#define FRAME_SIZE_625_50 	12 * 150 * 80
+
 /** To conserve resources, we maintain a stack of dv decoders.
 */
 
@@ -129,9 +134,9 @@ struct producer_libdv_s
 static int producer_get_frame( mlt_producer parent, mlt_frame_ptr frame, int index );
 static void producer_close( mlt_producer parent );
 
-static int producer_collect_info( producer_libdv this );
+static int producer_collect_info( producer_libdv this, mlt_profile profile );
 
-mlt_producer producer_libdv_init( char *filename )
+mlt_producer producer_libdv_init( mlt_profile profile, mlt_service_type type, const char *id, char *filename )
 {
 	producer_libdv this = calloc( sizeof( struct producer_libdv_s ), 1 );
 
@@ -156,14 +161,15 @@ mlt_producer producer_libdv_init( char *filename )
 			 strncasecmp( strrchr( filename, '.' ), ".mov", 4 ) == 0 ) )
 		{
 			// Load via an alternative mechanism
-			this->alternative = mlt_factory_producer( "kino", filename );
+			mlt_profile profile = mlt_service_profile( MLT_PRODUCER_SERVICE( producer ) );
+			this->alternative = mlt_factory_producer( profile, "kino", filename );
 
 			// If it's unavailable, then clean up
 			if ( this->alternative == NULL )
 				destroy = 1;
 			else
 				mlt_properties_pass( properties, MLT_PRODUCER_PROPERTIES( this->alternative ), "" );
-			this->is_pal = mlt_properties_get_int( properties, "fps" ) == 25;
+			this->is_pal = ( ( int ) mlt_producer_get_fps( producer ) ) == 25;
 		}
 		else
 		{
@@ -171,7 +177,7 @@ mlt_producer producer_libdv_init( char *filename )
 			this->fd = open( filename, O_RDONLY );
 
 			// Collect info
-			if ( this->fd == -1 || !producer_collect_info( this ) )
+			if ( this->fd == -1 || !producer_collect_info( this, profile ) )
 				destroy = 1;
 		}
 
@@ -195,7 +201,6 @@ static int read_frame( int fd, uint8_t* frame_buf, int *isPAL )
 	if ( result )
 	{
 		*isPAL = ( frame_buf[3] & 0x80 );
-
 		if ( *isPAL )
 		{
 			int diff = FRAME_SIZE_625_50 - FRAME_SIZE_525_60;
@@ -206,7 +211,7 @@ static int read_frame( int fd, uint8_t* frame_buf, int *isPAL )
 	return result;
 }
 
-static int producer_collect_info( producer_libdv this )
+static int producer_collect_info( producer_libdv this, mlt_profile profile )
 {
 	int valid = 0;
 
@@ -240,8 +245,8 @@ static int producer_collect_info( producer_libdv this )
 			this->frames_in_file = this->file_size / this->frame_size;
 
 			// Calculate default in/out points
-			double fps = this->is_pal ? 25 : 30000.0 / 1001.0;
-			if ( mlt_properties_get_double( properties, "fps" ) == fps )
+			int fps = 1000 * ( this->is_pal ? 25 : ( 30000.0 / 1001.0 ) );
+			if ( ( int )( mlt_profile_fps( profile ) * 1000 ) == fps )
 			{
 				if ( this->frames_in_file > 0 )
 				{
@@ -259,6 +264,16 @@ static int producer_collect_info( producer_libdv this )
 			dv_parse_header( dv_decoder, dv_data );
 			mlt_properties_set_double( properties, "aspect_ratio", 
 				dv_format_wide( dv_decoder ) ? ( this->is_pal ? 118.0/81.0 : 40.0/33.0 ) : ( this->is_pal ? 59.0/54.0 : 10.0/11.0 ) );
+			mlt_properties_set_double( properties, "source_fps", this->is_pal ? 25 : ( 30000.0 / 1001.0 ) );
+			mlt_properties_set_int( properties, "meta.media.nb_streams", 2 );
+			mlt_properties_set_int( properties, "video_index", 0 );
+			mlt_properties_set( properties, "meta.media.0.stream.type", "video" );
+			mlt_properties_set( properties, "meta.media.0.codec.name", "dvvideo" );
+			mlt_properties_set( properties, "meta.media.0.codec.long_name", "DV (Digital Video)" );
+			mlt_properties_set_int( properties, "audio_index", 1 );
+			mlt_properties_set( properties, "meta.media.1.stream.type", "audio" );
+			mlt_properties_set( properties, "meta.media.1.codec.name", "pcm_s16le" );
+			mlt_properties_set( properties, "meta.media.1.codec.long_name", "signed 16-bit little-endian PCM" );
 
 			// Return the decoder
 			dv_decoder_return( dv_decoder );
@@ -427,7 +442,7 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 		data = mlt_pool_alloc( FRAME_SIZE_625_50 );
 
 		// Create an empty frame
-		*frame = mlt_frame_init( );
+		*frame = mlt_frame_init( MLT_PRODUCER_SERVICE( producer ) );
 
 		// Seek and fetch
 		if ( this->fd != 0 && 
@@ -470,6 +485,8 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 		// Update other info on the frame
 		mlt_properties_set_int( properties, "width", 720 );
 		mlt_properties_set_int( properties, "height", this->is_pal ? 576 : 480 );
+		mlt_properties_set_int( properties, "real_width", 720 );
+		mlt_properties_set_int( properties, "real_height", this->is_pal ? 576 : 480 );
 		mlt_properties_set_int( properties, "top_field_first", !this->is_pal ? 0 : ( data[ 5 ] & 0x07 ) == 0 ? 0 : 1 );
 	
 		// Parse the header for meta info
@@ -482,14 +499,18 @@ static int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int i
 		mlt_properties_set_int( properties, "frequency", dv_decoder->audio->frequency );
 		mlt_properties_set_int( properties, "channels", dv_decoder->audio->num_channels );
 
-		// Hmm - register audio callback
-		mlt_frame_push_audio( *frame, producer_get_audio );
+		// Register audio callback
+		if ( mlt_properties_get_int( MLT_PRODUCER_PROPERTIES( producer ), "audio_index" ) > 0 )
+			mlt_frame_push_audio( *frame, producer_get_audio );
 	
-		// Push the quality string
-		mlt_frame_push_service( *frame, mlt_properties_get( MLT_PRODUCER_PROPERTIES( producer ), "quality" ) );
-	
-		// Push the get_image method on to the stack
-		mlt_frame_push_get_image( *frame, producer_get_image );
+		if ( mlt_properties_get_int( MLT_PRODUCER_PROPERTIES( producer ), "video_index" ) > -1 )
+		{
+			// Push the quality string
+			mlt_frame_push_service( *frame, mlt_properties_get( MLT_PRODUCER_PROPERTIES( producer ), "quality" ) );
+
+			// Push the get_image method on to the stack
+			mlt_frame_push_get_image( *frame, producer_get_image );
+		}
 	
 		// Return the decoder
 		dv_decoder_return( dv_decoder );
