@@ -19,30 +19,13 @@
  */
 
 #include <string.h>
+#include <framework/mlt.h>
 
-#include "producer_inigo.h"
+extern mlt_producer producer_inigo_file_init( mlt_profile profile, mlt_service_type type, const char *id, char *arg );
+extern mlt_producer producer_inigo_init( mlt_profile profile, mlt_service_type type, const char *id, char **argv );
 
-void *mlt_create_producer( char *id, void *arg )
+MLT_REPOSITORY
 {
-	if ( !strcmp( id, "inigo_file" ) )
-		return producer_inigo_file_init( arg );
-	if ( !strcmp( id, "inigo" ) )
-		return producer_inigo_init( arg );
-	return NULL;
+	MLT_REGISTER( producer_type, "inigo", producer_inigo_init );
+	MLT_REGISTER( producer_type, "inigo_file", producer_inigo_file_init );
 }
-
-void *mlt_create_filter( char *id, void *arg )
-{
-	return NULL;
-}
-
-void *mlt_create_transition( char *id, void *arg )
-{
-	return NULL;
-}
-
-void *mlt_create_consumer( char *id, void *arg )
-{
-	return NULL;
-}
-
