@@ -515,10 +515,10 @@ void RIFFFile::ParseChunk( int parent )
 
 	/* Check whether it is a LIST. If so, let ParseList deal with it */
 
-	read( fd, &type, sizeof( type ) );
+	fail_if( read( fd, &type, sizeof( type ) ) != sizeof( type ));
 	if ( type == make_fourcc( "LIST" ) )
 	{
-		typesize = -sizeof( type );
+		typesize = (int) -sizeof( type );
 		fail_if( lseek( fd, typesize, SEEK_CUR ) == ( off_t ) - 1 );
 		ParseList( parent );
 	}
