@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xineutils.h 1260 2008-12-12 07:53:04Z ddennedy $
+ * $Id: xineutils.h 1386 2009-04-10 07:18:41Z ddennedy $
  *
  */
 #ifndef XINEUTILS_H
@@ -659,13 +659,13 @@ void xine_strdupa(char *dest, char *src);
 #ifdef HAVE_STRPBRK
 #define xine_strpbrk strpbrk
 #else
-static inline char *_private_strpbrk(const char *s, const char *accept) {
+static inline const char *_private_strpbrk(const char *s, const char *accept) {
 
   while(*s != '\0') {
     const char *a = accept;
     while(*a != '\0')
       if(*a++ == *s)
-	return(char *) s;
+	return s;
     ++s;
   }
 
